"""This module provides functionality for defining your site and registrering applications"""
import pathlib
from functools import wraps
from typing import Callable, Dict, List, Optional

import param
from panel.template.base import BasicTemplate

from awesome_panel_extensions.site.application import Application
from awesome_panel_extensions.site.template import TemplateGenerator


class Site(param.Parameterized):
    """The Site provides meta data and functionality for registrering application meta data and
    views"""

    applications = param.List(doc="The list of applications to include in the site", constant=True)
    authors = param.List(doc="The list of authors contributing to the site", constant=True)
    css_path = param.ClassSelector(doc="A path to custom css", class_=pathlib.Path)
    js_path = param.ClassSelector(doc="A path to custom js", class_=pathlib.Path)

    def __init__(self, **params):
        if "applications" not in params:
            params["applications"] = []
        if "authors" not in params:
            params["authors"] = []

        super().__init__(**params)

        self._template_generator = TemplateGenerator(css_path=self.css_path, js_path=self.js_path)

    def create_application(  # pylint: disable=too-many-arguments
        self,
        url: str,
        name: str,
        introduction: str,
        description: str,
        author: str,
        thumbnail_url: str,
        code_url: str = "",
        documentation_url: str = "",
        gif_url: str = "",
        mp4_url: str = "",
        youtube_url: str = "",
        tags: Optional[List] = None,
    ) -> Application:
        """A Site consists of meta data, Resources, Applications and Routes.

        Args:
            url (str): The base url of the Site. For example 'https://awesome-panel.org'
            name (str): The name of the site. For example 'Awesome Panel'.
            introduction (str): A short description of the site.
            description (str): A longer description of the site.
            author (str): The name of the Author of the site.
            thumbnail_url (str): A Thumbnail visualising the site.
            code_url (str, optional): [description]. Defaults to "".
            documentation_url (str, optional): [description]. Defaults to "".
            gif_url (str, optional): [description]. Defaults to "".
            mp4_url (str, optional): [description]. Defaults to "".
            youtube_url (str, optional): [description]. Defaults to "".
            tags (Optional[List], optional): [description]. Defaults to None.

        Raises:
            ValueError: [description]

        Returns:
            Application: [description]
        """
        if tags is None:
            tags = []

        # pylint: disable=unsubscriptable-object, not-an-iterable
        author_list = [auth for auth in self.authors if auth.name == author]
        if author_list:
            author_ = author_list[0]
        else:
            raise ValueError(f"Error. Author '{author}' is not in the list of site authors!")

        return Application(
            url=url,
            name=name,
            introduction=introduction,
            description=description,
            author=author_,
            thumbnail_url=thumbnail_url,
            tags=tags,
            category="Application",
            documentation_url=documentation_url,
            code_url=code_url,
            gif_url=gif_url,
            mp4_url=mp4_url,
            youtube_url=youtube_url,
        )

    def add(
        self,
        application=None,
    ):
        """Registers you application meta data and view
        >>> from awesome_panel_extensions.site import Author
        >>> from awesome_panel_extensions.site import Site, Application
        >>> site = Site(name="awesome-panel.org")
        >>> marc_skov_madsen = Author(
        ...     name="Marc Skov Madsen",
        ...     url="https://datamodelsanalytics.com",
        ...     avatar_url="https://avatars0.githubusercontent.com/u/42288570",
        ...     twitter_url="https://twitter.com/MarcSkovMadsen",
        ...     linkedin_url="https://www.linkedin.com/in/marcskovmadsen/",
        ...     github_url="https://github.com/MarcSkovMadsen",
        ... )
        >>> site.authors.append(marc_skov_madsen)
        >>> application = site.create_application(
        ...     url="home",
        ...     name="Home",
        ...     author="Marc Skov Madsen",
        ...     description="The home page of awesome-panel.org.",
        ...     thumbnail_url="",
        ...     documentation_url="",
        ...     code_url="",
        ...     gif_url="",
        ...     mp4_url="",
        ...     youtube_url="",
        ...     tags=["Site"],
        ... )
        >>> @site.add(application)
        ... def view():
        ...     return pn.pane.Markdown("# Home")
        >>> site.applications
        [Home]
        >>> site.routes
        {'home': <function view at...>}
        """
        # pylint: disable=unsupported-assignment-operation
        if not application in self.applications: # pylint: disable=unsupported-membership-test
            self.applications.append(application)

        def inner_function(view):
            @wraps(view)
            def wrapper(*args, **kwargs):
                template = view(*args, **kwargs)
                if (
                    isinstance(template, BasicTemplate)
                    and template.title == template.param.title.default
                ):
                    if not self.name == application.name:
                        template.title = application.name
                    else:
                        template.title = ""
                self.register_post_view(template=template, application=application)
                return template

            application.view = wrapper
            return wrapper

        return inner_function

    def register_post_view(self, template: BasicTemplate, application: Application):
        """Can be overriden in child classes if you want to add something to the template after
        the view is run"""

    @property
    def routes(self) -> Dict[str, Callable]:
        """Returns a dictionary with the url as key and the view as the value

        Returns:
            Dict[str, Callable]: [description]
        """
        # pylint: disable=not-an-iterable
        return {app.url: app.view for app in self.applications}

    def create_template(  # pylint: disable=too-many-arguments, too-complex
        self,
        template: Optional[str] = None,
        theme: Optional[str] = None,
        **params,
    ) -> BasicTemplate:
        """Returns a BasicTemplate

        Args:
            template (str, optional): The name of the template. Defaults to TEMPLATE.
            theme (str, optional): The name of the theme. Defaults to THEME.
            **params: Optional template parameters

        Returns:
            BasicTemplate: The specified Template
        """
        params["site"] = params.get("site", self.name)
        return self._template_generator.get_template(
            template=template,
            theme=theme,
            **params,
        )

    def get_application(self, name: str) -> Optional[Application]:
        """Returns the application with the specified name

        Args:
            name (str): [description]

        Returns:
            Optional[Application]: [description]
        """
        # pylint: disable=not-an-iterable
        _app = [app for app in self.applications if app.name == name]
        if _app:
            return _app[0]
        return None



"""The Icon can be used to add SVG based icons inline to buttons, menus etc."""
import panel as pn

# See https://github.com/holoviz/panel/issues/1586 for motivation, possibilities and requirements.
import param

from awesome_panel_extensions.bokeh_extensions.icon import Icon as _BkIcon

_CSS = """
@-ms-keyframes spin {
    from {
        -ms-transform: rotate(0deg);
    }
    to {
        -ms-transform: rotate(360deg);
    }
}
@-moz-keyframes spin {
    from {
        -moz-transform: rotate(0deg);
    }
    to {
        -moz-transform: rotate(360deg);
    }
}
@-webkit-keyframes spin {
    from {
        -webkit-transform: rotate(0deg);
    }
    to {
        -webkit-transform: rotate(360deg);
    }
}
@keyframes spin {
    from {
        transform: rotate(0deg);
    }
    to {
        transform: rotate(360deg);
    }
}
    """
# Please remove .bk-btn-light above when https://github.com/bokeh/bokeh/issues/10505 has been
# solved
pn.config.raw_css.append(_CSS)


class Icon(param.Parameterized):
    # pylint: disable=line-too-long
    """The Icon can be used to add SVG based icons inline to buttons, menus etc.

    >>> Icon(
    ...    name="Github",
    ...    value='<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 19 20"><path d="M6.71154 17.0776C2.09615 18.4906 2.09615 14.7226 0.25 14.2517L6.71154 17.0776ZM13.1731 19.9036V16.2581C13.2077 15.8089 13.1482 15.3574 12.9986 14.9335C12.849 14.5096 12.6127 14.123 12.3054 13.7995C15.2038 13.4698 18.25 12.3489 18.25 7.20563C18.2498 5.89046 17.754 4.62572 16.8654 3.67319C17.2862 2.52257 17.2564 1.25074 16.7823 0.121918C16.7823 0.121918 15.6931 -0.207776 13.1731 1.51605C11.0574 0.930913 8.82722 0.930913 6.71154 1.51605C4.19154 -0.207776 3.10231 0.121918 3.10231 0.121918C2.62819 1.25074 2.59844 2.52257 3.01923 3.67319C2.12396 4.63279 1.62771 5.90895 1.63462 7.23389C1.63462 12.3394 4.68077 13.4604 7.57923 13.8278C7.27554 14.148 7.04132 14.5299 6.89182 14.9486C6.74233 15.3674 6.6809 15.8135 6.71154 16.2581V19.9036"></path></svg>',
    ...    fill_color="#E1477E",
    ...    spin_duration=2000,
    ... )
    Icon(...
    """
    # pylint: enable=line-too-long
    name = param.String(
        default=None,
        constant=False,
        doc="""The name of the icon. We will append 'icon' and 'icon-{name}'
        to enable custom styling""",
    )
    value = param.String(
        doc="""
        A html string defining the icon.
        """
    )
    size = param.Number(
        default=1.0,
        bounds=(0.0, None),
        step=0.1,
        doc="The size in em units, i.e. a multiplier of the current font-size.",
    )
    fill_color = param.String(
        default="currentColor",
        doc="""The fill color of the Icon. Any valid css color like '#eb4034', 'rgb(235, 64, 52)'
        or 'currentColor'. Default is 'currentColor' which is the color of the surrounding text""",
    )
    # For CSS Spin See https://codepen.io/eveness/pen/BjLaoa
    spin_duration = param.Integer(
        default=0,
        bounds=(0, None),
        doc="""The spin duration in miliseconds.
        If greater than 0 the Icon will do a spinning animation. Defaults to 0""",
    )

    _bk_icon = param.ClassSelector(class_=_BkIcon)

    def __init__(self, **params):
        super().__init__(**params)

        self._bk_icon = _BkIcon(
            label=self.name,
            text=self.value,
            size=self.size,
            fill_color=self.fill_color,
            spin_duration=self.spin_duration,
        )

    @param.depends("name", watch=True)
    def _update_label(self, *_):
        self._bk_icon.label = self.name

    @param.depends("value", watch=True)
    def _update_value(self, *_):
        self._bk_icon.text = self.value

    @param.depends("size", watch=True)
    def _update_size(self, *_):
        self._bk_icon.size = self.size

    @param.depends("fill_color", watch=True)
    def _update_fill_color(self, *_):
        self._bk_icon.fill_color = self.fill_color

    @param.depends("spin_duration", watch=True)
    def _update_spin_duration(self, *_):
        self._bk_icon.spin_duration = self.spin_duration



"""This module implements the TitleComponent which provides a nice looking Title Bar to the
Designer"""
import panel as pn
import param

from awesome_panel_extensions import assets


class TitleComponent(param.Parameterized):
    """The TitleComponent provides a nice looking Title Bar to the Designer"""

    title = param.String("Panel Designer")
    title_url = param.String("https://panel.holoviz.org")
    subtitle = param.String("awesome-panel.org")
    subtitle_url = param.String("https://awesome-panel.org")
    logo_url = param.String(assets.SPINNER_PANEL_STATIC_LIGHT_400_340)
    logo_spinning_url = param.String(assets.SPINNER_PANEL_BREATH_LIGHT_400_340)
    spinning = param.Boolean(False)
    start_spinning = param.Action(label="Start Spinning")
    stop_spinning = param.Action(label="Stop Spinning")

    spinner_pane = param.Parameter()
    title_pane = param.ClassSelector(class_=pn.pane.HTML)
    view = param.ClassSelector(class_=pn.Row)

    def __init__(
        self,
        **params,
    ):
        params["start_spinning"] = self._start_spinning
        params["stop_spinning"] = self._stop_spinning

        super().__init__(**params)

        self.title_pane = pn.pane.HTML(margin=(5, 5, 10, 15), sizing_mode="stretch_width")
        self._update_title_pane()

        self.spinner_pane = pn.pane.HTML(
            sizing_mode="fixed", width=70, height=70, align="center", margin=(5, 50, 10, 5)
        )
        self._updatespinner_pane()
        self.view = pn.Row(
            self.title_pane,
            self.spinner_pane,
            name="TitleComponent.view",
            sizing_mode="stretch_width",
            css_classes=["designer-title-component"],
        )

    @param.depends(
        "title",
        "title_url",
        "subtitle",
        "subtitle_url",
        "logo_url",
        "logo_spinning_url",
        watch=True,
    )
    def _update_title_pane(self):
        html = f"""\
<h1><a href="{self.title_url}" target="_blank">{self.title}</a></h1>
<p>By <strong><a href="{self.subtitle_url}" target="_blank">{self.subtitle}</a></strong></p>
"""
        self.title_pane.object = html

    @param.depends("spinning", watch=True)
    def _updatespinner_pane(self):
        if self.spinning:
            url = self.logo_spinning_url
        else:
            url = self.logo_url

        html = f"""
<a href="https://panel.holoviz.org" target="_blank">
<img src="{url}" style="max-height:100%;max-width:100%"></img>
</a>
"""

        # html = f"<img src='{url}' style='width:100%'></img>"
        self.spinner_pane.object = html

    def _start_spinning(self, _=None):
        self.spinning = True

    def _stop_spinning(self, _=None):
        self.spinning = False

    def __repr__(self):
        return f"TitleComponent({self.name})"

    def __str__(self):
        return f"TitleComponent({self.name})"



"""The FastLiteratAreaInput corresponds to the Panel LiteralInput."""
from panel.widgets.input import LiteralInput

from awesome_panel_extensions.bokeh_extensions.fast.fast_textarea_input import (
    FastTextAreaInput as _BkFastTextAreaInput,
)
from awesome_panel_extensions.frameworks.fast.fast_textarea_input import _FastTextAreaInputMixin


class FastLiteralAreaInput(_FastTextAreaInputMixin, LiteralInput):
    """The FastLiteratAreaInput corresponds to the Panel LiteralInput."""

    _widget_type = _BkFastTextAreaInput

    _rename = {**LiteralInput._rename}



"""The FastGalleryTemplate can be used to show case your applications in a nice way"""
import pathlib
from typing import List

import panel as pn
from panel import Template

from awesome_panel_extensions.site import Resource

ROOT = pathlib.Path(__file__).parent
CSS = (ROOT / "fast_gallery_template.css").read_text()
JS = (ROOT / "fast_gallery_template.js").read_text()
TEMPLATE = (ROOT / "fast_gallery_template.html").read_text()

pn.config.raw_css.append(CSS)


class FastGalleryTemplate(Template):
    """The FastGalleryTemplate can be used to show case your applications in a nice way"""

    def __init__(  # pylint: disable=too-many-arguments
        self,
        items: List[Resource],
        site_name: str = "Awesome Panel",
        site_url: str = "https://awesome-panel.org",
        name: str = "Gallery",
        url: str = "https://awesome-panel.org/gallery",
        description: str = "",
        background_image_url: str = "",
        target: str = "_self",
    ):
        """The FastGalleryTemplate can be used to show case your applications in a nice way

        TO BE IMPROVED

        Args:
            items (List[Resource]): [description]
            site_name (str, optional): [description]. Defaults to "Awesome Panel".
            site_url (str, optional): [description]. Defaults to "https://awesome-panel.org".
            name (str, optional): [description]. Defaults to "Gallery".
            url (str, optional): [description]. Defaults to "https://awesome-panel.org/gallery".
            description (str, optional): [description]. Defaults to "".
            background_image_url (str, optional): [description]. Defaults to "".
            target (str, optional): [description]. Defaults to "_self".
        """
        super().__init__(template=TEMPLATE)
        self.add_variable("title", site_name + "|" + name)
        self.add_variable("site_name", site_name)
        self.add_variable("site_url", site_url)
        self.add_variable("name", name)
        self.add_variable("url", url)
        self.add_variable("description", description)
        self.add_variable("background_image_url", background_image_url)
        self.add_variable("items", items)
        self.add_variable("gallery_js", JS)

        if target not in ["_blank", "_parent", "_top", "_self"]:
            target = "_self"
        self.add_variable("target", target)



"""The Resource contains meta data like name, description and url"""
from typing import List

import markdown
import panel as pn
import param

from awesome_panel_extensions.assets.svg_icons import ICONS

from . import category
from .author import Author
from .base_model import BaseModel

MARKDOWN_EXTENSIONS = ["extra", "smarty", "codehilite"]
STYLE = """
.pnx-resource img.pnx-avatar {
    height: 2em;
    width: 2em;
    margin-left: 0.5em;
}
.pnx-resource svg.pnx-icon {
    height: 1.5em;
    width: 1.5em;
    margin-left: 0.5em;
    vertical-align: middle;
}
.pnx-resource a {
    text-decoration: none;
}
"""
if not STYLE in pn.config.raw_css:
    pn.config.raw_css.append(STYLE)


class Resource(BaseModel):
    """The Resource contains meta data like name, description and url"""

    name = param.String(doc="The name")
    introduction = param.String(doc="A short text description.")
    description = param.String(doc="A longer description. Can contain Markdown and HTML")
    author = param.ClassSelector(class_=Author)
    url = param.String(doc="A unique, identifying link.")
    thumbnail_url = param.String(doc="A link to a thumbnail image visualizing the resource.")

    tags = param.List(
        class_=str,
        doc="""A list of tags like 'machine-learning', 'panel', 'holoviews'. Don't use spaces in the
        tag.""",
    )
    category = param.ObjectSelector(default=category.NOT_AVAILABLE, objects=category.ALL)

    documentation_url = param.String(doc="A link to the documentation.")
    code_url = param.String(doc="A link to the source code.")
    mp4_url = param.String(doc="A link to a mp4 video.")
    youtube_url = param.String(doc="A link to a youtube video.")
    gif_url = param.String(doc="A link to a .gif video")

    all: List["Resource"] = []

    def __init__(self, **params):
        super().__init__(**params)

        self.all.append(self)

    @staticmethod
    def _get_url_icon_html(
        title,
        url,
    ):
        return (
            f"""<a title="{ title }" appearance="hypertext" href="{ url }" target="_blank">"""
            f"""{ ICONS[title] }</a>"""
        )

    @staticmethod
    def _markdown_to_html(text: str) -> str:
        return markdown.markdown(text, extensions=MARKDOWN_EXTENSIONS, output_format="html5")

    def intro_section(self) -> pn.pane.HTML:
        """An panel with a text introduction to the Resource

        Returns:
            pn.pane.HTML: The Intro Section panel.
        """
        return pn.pane.HTML(self._repr_html_())

    def _repr_html_(self):
        description = self._markdown_to_html(self.description)
        html = f"""<div class="pnx-resource">
        <h1 class="pnx-header">{ self.name }</h1>
        <p>{ description }</p>
        """
        if self.author:
            # pylint: disable=protected-access
            html += f"""<p><strong>Authors:</strong>{ self.author._repr_html_() }</p>"""

        if self.code_url:
            code = self._get_url_icon_html("code", self.code_url)
            html += "<p><strong>Code:</strong>" + code + "</p>"

        resources = ""
        if self.documentation_url:
            resources += self._get_url_icon_html("doc", self.documentation_url)
        if self.gif_url:
            resources += self._get_url_icon_html("gif", self.gif_url)
        if self.mp4_url:
            resources += self._get_url_icon_html("mp4", self.mp4_url)
        if self.youtube_url:
            resources += self._get_url_icon_html("youtube", self.youtube_url)

        if resources:
            html += "<p><strong>Resources:</strong>" + resources + "</p>"

        tags = ", #".join(self.tags)
        if tags:
            html += "<p><strong>Tags:</strong> #" + tags.lower() + "</p>"

        html += "</div>"
        return html

        # author = _to_avatar_icon(resource.author)



"""The FastButton extends the Panel Button to a Fast Design Framework Button.

It is built on the the fast-button web component. The component supports several visual apperances
(accent, lightweight, neutral, outline, stealth).

For more information view the [component specification]\
(https://github.com/microsoft/fast/tree/master/packages/web-components/fast-foundation/\
src/button/button.spec.md).

See also https://explore.fast.design/components/fast-button.
    """
import panel as pn
import param  # pylint: disable=wrong-import-order
from panel.widgets import Button

from awesome_panel_extensions.bokeh_extensions.fast.fast_button import FastButton as _BkFastButton

FAST_BUTTON_APPEARENCES = [
    "accent",
    "lightweight",
    "neutral",
    "outline",
    "stealth",
]
DEFAULT_FAST_BUTTON_APPEARANCE = "neutral"
BUTTON_TYPE_TO_APPEARANCE = {
    "default": "neutral",
    "primary": "accent",
    "success": "outline",
    "warning": "accent",
    "danger": "accent",
}


class FastButton(Button):
    """The FastButton extends the Panel Button into the Fast Design Framework.

It is built on the the fast-button web component. The component supports several visual apperances
(accent, lightweight, neutral, outline, stealth).

For more information view the [component specification]\
(https://github.com/microsoft/fast/tree/master/packages/web-components/fast-foundation/\
src/button/button.spec.md).

See also https://explore.fast.design/components/fast-button.
    """

    appearance = param.ObjectSelector(
        default=DEFAULT_FAST_BUTTON_APPEARANCE,
        objects=FAST_BUTTON_APPEARENCES,
        doc="""Determines the appearance of the button. One of `accent`, `lightweight`, `neutral`,
        `outline` or `stealth`. Defaults to neutral""",
        allow_None=True,
    )
    autofocus = param.Boolean(
        default=False,
        doc="""The autofocus attribute. Defaults to `False`""",
    )

    height = param.Integer(default=31, bounds=(0, None))

    _widget_type = _BkFastButton

    _rename = {
        **pn.widgets.Button._rename,  # pylint: disable=protected-access
    }

    def __init__(self, **params):
        if "button_type" in params and "appearance" not in params:
            params["appearance"] = BUTTON_TYPE_TO_APPEARANCE[params["button_type"]]
        super().__init__(**params)

    @param.depends("button_type", watch=True)
    def _update_accent(self, *_):
        self.appearance = BUTTON_TYPE_TO_APPEARANCE[self.button_type]



"""A collection of nice panes to use"""

import panel as pn


class Code(pn.pane.Markdown):
    """A HTML code block"""

    def __init__(
        self,
        code: str = "",
        language: str = "python",
        sizing_mode="stretch_width",
        **kwargs,
    ):
        """A HTML code block"""
        code_markdown = f"""
```{language}
{code}
```
"""
        super().__init__(
            code_markdown,
            sizing_mode=sizing_mode,
            **kwargs,
        )



"""The ColorPalette is inspired by the [Material Design Color Palette]\
    (https://material.io/resources/color/#!/?view.left=0&view.right=0&primary.color=9C27B0)

It provides

- functionality to create and edit a Color Palette similar to the Material Design Color Palette
- functionality to calculate a palette (close to) the Material Design Color Palette based on the
selection of one color
"""
import panel as pn
import param

from . import color
from .color_utils import compute_colors, is_dark

COLOR_PARAMETERS = [
    "color_50",
    "color_100",
    "color_200",
    "color_300",
    "color_400",
    "color_500",
    "color_600",
    "color_700",
    "color_800",
    "color_900",
    "color_a100",
    "color_a200",
    "color_a400",
    "color_a700",
]
CONTRAST_PARAMETERS = [
    "contrast_50",
    "contrast_100",
    "contrast_200",
    "contrast_300",
    "contrast_400",
    "contrast_500",
    "contrast_600",
    "contrast_700",
    "contrast_800",
    "contrast_900",
    "contrast_a100",
    "contrast_a200",
    "contrast_a400",
    "contrast_a700",
]
COLOR_AND_CONTRAST_PARAMETERS = COLOR_PARAMETERS + CONTRAST_PARAMETERS


class ColorPalette(param.Parameterized):  # pylint: disable=too-many-instance-attributes
    """A Material Design Color Palette """

    color_50 = param.Color(default="#f3e5f6", precedence=0.0)
    color_100 = param.Color(default="#e1bee7", precedence=0.1)
    color_200 = param.Color(default="#ce93d8", precedence=0.2)
    color_300 = param.Color(default="#ba68c8", precedence=0.3)
    color_400 = param.Color(default="#ab47bc", precedence=0.4)
    color_500 = param.Color(default="#9c27b0", precedence=0.5)
    color_600 = param.Color(default="#9423a9", precedence=0.6)
    color_700 = param.Color(default="#8a1da0", precedence=0.7)
    color_800 = param.Color(default="#801797", precedence=0.8)
    color_900 = param.Color(default="#6e0e87", precedence=0.9)
    color_a100 = param.Color(default="#efb8ff", precedence=1.1)
    color_a200 = param.Color(default="#e485ff", precedence=1.2)
    color_a400 = param.Color(default="#d852ff", precedence=1.4)
    color_a700 = param.Color(default="#d238ff", precedence=1.7)

    contrast_50 = param.ObjectSelector(
        default=color.DARK_PRIMARY_TEXT, objects=color.PRIMARY_TEXTS, precedence=0.0
    )
    contrast_100 = param.ObjectSelector(
        default=color.DARK_PRIMARY_TEXT, objects=color.PRIMARY_TEXTS, precedence=0.1
    )
    contrast_200 = param.ObjectSelector(
        default=color.DARK_PRIMARY_TEXT, objects=color.PRIMARY_TEXTS, precedence=0.2
    )
    contrast_300 = param.ObjectSelector(
        default=color.DARK_PRIMARY_TEXT, objects=color.PRIMARY_TEXTS, precedence=0.3
    )
    contrast_400 = param.ObjectSelector(
        default=color.LIGHT_PRIMARY_TEXT, objects=color.PRIMARY_TEXTS, precedence=0.4
    )
    contrast_500 = param.ObjectSelector(
        default=color.LIGHT_PRIMARY_TEXT, objects=color.PRIMARY_TEXTS, precedence=0.5
    )
    contrast_600 = param.ObjectSelector(
        default=color.LIGHT_PRIMARY_TEXT, objects=color.PRIMARY_TEXTS, precedence=0.6
    )
    contrast_700 = param.ObjectSelector(
        default=color.LIGHT_PRIMARY_TEXT, objects=color.PRIMARY_TEXTS, precedence=0.7
    )
    contrast_800 = param.ObjectSelector(
        default=color.LIGHT_PRIMARY_TEXT, objects=color.PRIMARY_TEXTS, precedence=0.8
    )
    contrast_900 = param.ObjectSelector(
        default=color.LIGHT_PRIMARY_TEXT, objects=color.PRIMARY_TEXTS, precedence=0.9
    )
    contrast_a100 = param.ObjectSelector(
        default=color.DARK_PRIMARY_TEXT, objects=color.PRIMARY_TEXTS, precedence=1.1
    )
    contrast_a200 = param.ObjectSelector(
        default=color.DARK_PRIMARY_TEXT, objects=color.PRIMARY_TEXTS, precedence=1.2
    )
    contrast_a400 = param.ObjectSelector(
        default=color.DARK_PRIMARY_TEXT, objects=color.PRIMARY_TEXTS, precedence=1.4
    )
    contrast_a700 = param.ObjectSelector(
        default=color.LIGHT_PRIMARY_TEXT, objects=color.PRIMARY_TEXTS, precedence=1.7
    )

    @param.depends("color_500", watch=True)
    def _update_colors_from_color_500(self):
        colors = compute_colors(self.color_500)

        self.color_50 = colors["50"]
        self.color_100 = colors["100"]
        self.color_200 = colors["200"]
        self.color_300 = colors["300"]
        self.color_400 = colors["400"]
        # self.color_500 = colors["500"]
        self.color_600 = colors["600"]
        self.color_700 = colors["700"]
        self.color_800 = colors["800"]
        self.color_900 = colors["900"]
        self.color_a100 = colors["A100"]
        self.color_a200 = colors["A200"]
        self.color_a400 = colors["A400"]
        self.color_a700 = colors["A700"]

        self.contrast_50 = (
            color.LIGHT_PRIMARY_TEXT if is_dark(self.color_50) else color.DARK_PRIMARY_TEXT
        )
        self.contrast_100 = (
            color.LIGHT_PRIMARY_TEXT if is_dark(self.color_100) else color.DARK_PRIMARY_TEXT
        )
        self.contrast_200 = (
            color.LIGHT_PRIMARY_TEXT if is_dark(self.color_200) else color.DARK_PRIMARY_TEXT
        )
        self.contrast_300 = (
            color.LIGHT_PRIMARY_TEXT if is_dark(self.color_300) else color.DARK_PRIMARY_TEXT
        )
        self.contrast_400 = (
            color.LIGHT_PRIMARY_TEXT if is_dark(self.color_400) else color.DARK_PRIMARY_TEXT
        )
        self.contrast_500 = (
            color.LIGHT_PRIMARY_TEXT if is_dark(self.color_500) else color.DARK_PRIMARY_TEXT
        )
        self.contrast_600 = (
            color.LIGHT_PRIMARY_TEXT if is_dark(self.color_600) else color.DARK_PRIMARY_TEXT
        )
        self.contrast_700 = (
            color.LIGHT_PRIMARY_TEXT if is_dark(self.color_700) else color.DARK_PRIMARY_TEXT
        )
        self.contrast_800 = (
            color.LIGHT_PRIMARY_TEXT if is_dark(self.color_800) else color.DARK_PRIMARY_TEXT
        )
        self.contrast_900 = (
            color.LIGHT_PRIMARY_TEXT if is_dark(self.color_900) else color.DARK_PRIMARY_TEXT
        )
        self.contrast_a100 = (
            color.LIGHT_PRIMARY_TEXT if is_dark(self.color_a100) else color.DARK_PRIMARY_TEXT
        )
        self.contrast_a200 = (
            color.LIGHT_PRIMARY_TEXT if is_dark(self.color_a200) else color.DARK_PRIMARY_TEXT
        )
        self.contrast_a400 = (
            color.LIGHT_PRIMARY_TEXT if is_dark(self.color_a400) else color.DARK_PRIMARY_TEXT
        )
        self.contrast_a700 = (
            color.LIGHT_PRIMARY_TEXT if is_dark(self.color_a700) else color.DARK_PRIMARY_TEXT
        )

    def editor(self) -> pn.Column:
        """Returns an editor from that enables the user to edit the Color Palette.

        Returns:
            [pn.Column]: An Color Palette Editor
        """
        return pn.Column(
            "## Color Palette Editor",
            pn.Param(
                self,
                show_name=False,
                parameters=COLOR_PARAMETERS,
            ),
        )

    def color_500_editor(self) -> pn.Column:
        """Returns an editor that enables the user to edit the Color 500.

        Returns:
            pn.Column: [description]
        """
        return pn.Column(
            "## Color Palette Editor",
            self.param.color_500,
            self.readonly_view(),
        )

    def to_html_table(self) -> str:
        """Returns a HTML string that shows the color palette"""
        return f"""\
<table style="width:100%;text-align:center"><tbody>
<tr style="background-color:{self.color_50};color:{self.contrast_50}"><td>50: </td><td>{self.color_50}</td></tr>
<tr style="background-color:{self.color_100};color:{self.contrast_100}"><td>100: </td><td>{self.color_100}</td></tr>
<tr style="background-color:{self.color_200};color:{self.contrast_200}"><td>200: </td><td>{self.color_200}</td></tr>
<tr style="background-color:{self.color_300};color:{self.contrast_300}"><td>300: </td><td>{self.color_300}</td></tr>
<tr style="background-color:{self.color_400};color:{self.contrast_400}"><td>400: </td><td>{self.color_400}</td></tr>
<tr style="background-color:{self.color_500};color:{self.contrast_500}"><td>500: </td><td>{self.color_500}</td></tr>
<tr style="background-color:{self.color_600};color:{self.contrast_600}"><td>600: </td><td>{self.color_600}</td></tr>
<tr style="background-color:{self.color_700};color:{self.contrast_700}"><td>700: </td><td>{self.color_700}</td></tr>
<tr style="background-color:{self.color_800};color:{self.contrast_800}"><td>800: </td><td>{self.color_800}</td></tr>
<tr style="background-color:{self.color_900};color:{self.contrast_900}"><td>900: </td><td>{self.color_900}</td></tr>
<tr style="background-color:{self.color_a100};color:{self.contrast_a100}"><td>A100: </td><td>{self.color_a100}</td></tr>
<tr style="background-color:{self.color_a200};color:{self.contrast_a200}"><td>A200: </td><td>{self.color_a200}</td></tr>
<tr style="background-color:{self.color_a400};color:{self.contrast_a400}"><td>A400: </td><td>{self.color_a400}</td></tr>
<tr style="background-color:{self.color_a700};color:{self.contrast_a700}"><td>A700: </td><td>{self.color_a700}</td></tr>
</tbody></table>"""

    def readonly_view(self) -> pn.Column:
        """Returns a read only view of the Color Palette

        Returns:
            pn.Column: A read only view
        """
        return pn.Column("## Color Palette", pn.pane.HTML(self.to_html_table()))

    def __eq__(self, other):
        return self.name == other.name


RED = ColorPalette(
    name="red",
    color_50="#ffebee",
    color_100="#ffcdd2",
    color_200="#ef9a9a",
    color_300="#e57373",
    color_400="#ef5350",
    color_500="#f44336",
    color_600="#e53935",
    color_700="#d32f2f",
    color_800="#c62828",
    color_900="#b71c1c",
    color_a100="#ff8a80",
    color_a200="#ff5252",
    color_a400="#ff1744",
    color_a700="#d50000",
    contrast_50=color.DARK_PRIMARY_TEXT,
    contrast_100=color.DARK_PRIMARY_TEXT,
    contrast_200=color.DARK_PRIMARY_TEXT,
    contrast_300=color.DARK_PRIMARY_TEXT,
    contrast_400=color.DARK_PRIMARY_TEXT,
    contrast_500=color.LIGHT_PRIMARY_TEXT,
    contrast_600=color.LIGHT_PRIMARY_TEXT,
    contrast_700=color.LIGHT_PRIMARY_TEXT,
    contrast_800=color.LIGHT_PRIMARY_TEXT,
    contrast_900=color.LIGHT_PRIMARY_TEXT,
    contrast_a100=color.DARK_PRIMARY_TEXT,
    contrast_a200=color.LIGHT_PRIMARY_TEXT,
    contrast_a400=color.LIGHT_PRIMARY_TEXT,
    contrast_a700=color.LIGHT_PRIMARY_TEXT,
)


PINK = ColorPalette(
    name="pink",
    color_50="#fce4ec",
    color_100="#f8bbd0",
    color_200="#f48fb1",
    color_300="#f06292",
    color_400="#ec407a",
    color_500="#e91e63",
    color_600="#d81b60",
    color_700="#c2185b",
    color_800="#ad1457",
    color_900="#880e4f",
    color_a100="#ff80ab",
    color_a200="#ff4081",
    color_a400="#f50057",
    color_a700="#c51162",
    contrast_50=color.DARK_PRIMARY_TEXT,
    contrast_100=color.DARK_PRIMARY_TEXT,
    contrast_200=color.DARK_PRIMARY_TEXT,
    contrast_300=color.DARK_PRIMARY_TEXT,
    contrast_400=color.DARK_PRIMARY_TEXT,
    contrast_500=color.LIGHT_PRIMARY_TEXT,
    contrast_600=color.LIGHT_PRIMARY_TEXT,
    contrast_700=color.LIGHT_PRIMARY_TEXT,
    contrast_800=color.LIGHT_PRIMARY_TEXT,
    contrast_900=color.LIGHT_PRIMARY_TEXT,
    contrast_a100=color.DARK_PRIMARY_TEXT,
    contrast_a200=color.LIGHT_PRIMARY_TEXT,
    contrast_a400=color.LIGHT_PRIMARY_TEXT,
    contrast_a700=color.LIGHT_PRIMARY_TEXT,
)


PURPLE = ColorPalette(
    name="purple",
    color_50="#f3e5f5",
    color_100="#e1bee7",
    color_200="#ce93d8",
    color_300="#ba68c8",
    color_400="#ab47bc",
    color_500="#9c27b0",
    color_600="#8e24aa",
    color_700="#7b1fa2",
    color_800="#6a1b9a",
    color_900="#4a148c",
    color_a100="#ea80fc",
    color_a200="#e040fb",
    color_a400="#d500f9",
    color_a700="#aa00ff",
    contrast_50=color.DARK_PRIMARY_TEXT,
    contrast_100=color.DARK_PRIMARY_TEXT,
    contrast_200=color.DARK_PRIMARY_TEXT,
    contrast_300=color.LIGHT_PRIMARY_TEXT,
    contrast_400=color.LIGHT_PRIMARY_TEXT,
    contrast_500=color.LIGHT_PRIMARY_TEXT,
    contrast_600=color.LIGHT_PRIMARY_TEXT,
    contrast_700=color.LIGHT_PRIMARY_TEXT,
    contrast_800=color.LIGHT_PRIMARY_TEXT,
    contrast_900=color.LIGHT_PRIMARY_TEXT,
    contrast_a100=color.DARK_PRIMARY_TEXT,
    contrast_a200=color.LIGHT_PRIMARY_TEXT,
    contrast_a400=color.LIGHT_PRIMARY_TEXT,
    contrast_a700=color.LIGHT_PRIMARY_TEXT,
)


DEEP_PURPLE = ColorPalette(
    name="deep_purple",
    color_50="#ede7f6",
    color_100="#d1c4e9",
    color_200="#b39ddb",
    color_300="#9575cd",
    color_400="#7e57c2",
    color_500="#673ab7",
    color_600="#5e35b1",
    color_700="#512da8",
    color_800="#4527a0",
    color_900="#311b92",
    color_a100="#b388ff",
    color_a200="#7c4dff",
    color_a400="#651fff",
    color_a700="#6200ea",
    contrast_50=color.DARK_PRIMARY_TEXT,
    contrast_100=color.DARK_PRIMARY_TEXT,
    contrast_200=color.DARK_PRIMARY_TEXT,
    contrast_300=color.LIGHT_PRIMARY_TEXT,
    contrast_400=color.LIGHT_PRIMARY_TEXT,
    contrast_500=color.LIGHT_PRIMARY_TEXT,
    contrast_600=color.LIGHT_PRIMARY_TEXT,
    contrast_700=color.LIGHT_PRIMARY_TEXT,
    contrast_800=color.LIGHT_PRIMARY_TEXT,
    contrast_900=color.LIGHT_PRIMARY_TEXT,
    contrast_a100=color.DARK_PRIMARY_TEXT,
    contrast_a200=color.LIGHT_PRIMARY_TEXT,
    contrast_a400=color.LIGHT_PRIMARY_TEXT,
    contrast_a700=color.LIGHT_PRIMARY_TEXT,
)


INDIGO = ColorPalette(
    name="indigo",
    color_50="#e8eaf6",
    color_100="#c5cae9",
    color_200="#9fa8da",
    color_300="#7986cb",
    color_400="#5c6bc0",
    color_500="#3f51b5",
    color_600="#3949ab",
    color_700="#303f9f",
    color_800="#283593",
    color_900="#1a237e",
    color_a100="#8c9eff",
    color_a200="#536dfe",
    color_a400="#3d5afe",
    color_a700="#304ffe",
    contrast_50=color.DARK_PRIMARY_TEXT,
    contrast_100=color.DARK_PRIMARY_TEXT,
    contrast_200=color.DARK_PRIMARY_TEXT,
    contrast_300=color.LIGHT_PRIMARY_TEXT,
    contrast_400=color.LIGHT_PRIMARY_TEXT,
    contrast_500=color.LIGHT_PRIMARY_TEXT,
    contrast_600=color.LIGHT_PRIMARY_TEXT,
    contrast_700=color.LIGHT_PRIMARY_TEXT,
    contrast_800=color.LIGHT_PRIMARY_TEXT,
    contrast_900=color.LIGHT_PRIMARY_TEXT,
    contrast_a100=color.DARK_PRIMARY_TEXT,
    contrast_a200=color.LIGHT_PRIMARY_TEXT,
    contrast_a400=color.LIGHT_PRIMARY_TEXT,
    contrast_a700=color.LIGHT_PRIMARY_TEXT,
)


BLUE = ColorPalette(
    name="blue",
    color_50="#e3f2fd",
    color_100="#bbdefb",
    color_200="#90caf9",
    color_300="#64b5f6",
    color_400="#42a5f5",
    color_500="#2196f3",
    color_600="#1e88e5",
    color_700="#1976d2",
    color_800="#1565c0",
    color_900="#0d47a1",
    color_a100="#82b1ff",
    color_a200="#448aff",
    color_a400="#2979ff",
    color_a700="#2962ff",
    contrast_50=color.DARK_PRIMARY_TEXT,
    contrast_100=color.DARK_PRIMARY_TEXT,
    contrast_200=color.DARK_PRIMARY_TEXT,
    contrast_300=color.DARK_PRIMARY_TEXT,
    contrast_400=color.DARK_PRIMARY_TEXT,
    contrast_500=color.LIGHT_PRIMARY_TEXT,
    contrast_600=color.LIGHT_PRIMARY_TEXT,
    contrast_700=color.LIGHT_PRIMARY_TEXT,
    contrast_800=color.LIGHT_PRIMARY_TEXT,
    contrast_900=color.LIGHT_PRIMARY_TEXT,
    contrast_a100=color.DARK_PRIMARY_TEXT,
    contrast_a200=color.LIGHT_PRIMARY_TEXT,
    contrast_a400=color.LIGHT_PRIMARY_TEXT,
    contrast_a700=color.LIGHT_PRIMARY_TEXT,
)


LIGHT_BLUE = ColorPalette(
    name="light_blue",
    color_50="#e1f5fe",
    color_100="#b3e5fc",
    color_200="#81d4fa",
    color_300="#4fc3f7",
    color_400="#29b6f6",
    color_500="#03a9f4",
    color_600="#039be5",
    color_700="#0288d1",
    color_800="#0277bd",
    color_900="#01579b",
    color_a100="#80d8ff",
    color_a200="#40c4ff",
    color_a400="#00b0ff",
    color_a700="#0091ea",
    contrast_50=color.DARK_PRIMARY_TEXT,
    contrast_100=color.DARK_PRIMARY_TEXT,
    contrast_200=color.DARK_PRIMARY_TEXT,
    contrast_300=color.DARK_PRIMARY_TEXT,
    contrast_400=color.DARK_PRIMARY_TEXT,
    contrast_500=color.LIGHT_PRIMARY_TEXT,
    contrast_600=color.LIGHT_PRIMARY_TEXT,
    contrast_700=color.LIGHT_PRIMARY_TEXT,
    contrast_800=color.LIGHT_PRIMARY_TEXT,
    contrast_900=color.LIGHT_PRIMARY_TEXT,
    contrast_a100=color.DARK_PRIMARY_TEXT,
    contrast_a200=color.DARK_PRIMARY_TEXT,
    contrast_a400=color.DARK_PRIMARY_TEXT,
    contrast_a700=color.LIGHT_PRIMARY_TEXT,
)


CYAN = ColorPalette(
    name="cyan",
    color_50="#e0f7fa",
    color_100="#b2ebf2",
    color_200="#80deea",
    color_300="#4dd0e1",
    color_400="#26c6da",
    color_500="#00bcd4",
    color_600="#00acc1",
    color_700="#0097a7",
    color_800="#00838f",
    color_900="#006064",
    color_a100="#84ffff",
    color_a200="#18ffff",
    color_a400="#00e5ff",
    color_a700="#00b8d4",
    contrast_50=color.DARK_PRIMARY_TEXT,
    contrast_100=color.DARK_PRIMARY_TEXT,
    contrast_200=color.DARK_PRIMARY_TEXT,
    contrast_300=color.DARK_PRIMARY_TEXT,
    contrast_400=color.DARK_PRIMARY_TEXT,
    contrast_500=color.LIGHT_PRIMARY_TEXT,
    contrast_600=color.LIGHT_PRIMARY_TEXT,
    contrast_700=color.LIGHT_PRIMARY_TEXT,
    contrast_800=color.LIGHT_PRIMARY_TEXT,
    contrast_900=color.LIGHT_PRIMARY_TEXT,
    contrast_a100=color.DARK_PRIMARY_TEXT,
    contrast_a200=color.DARK_PRIMARY_TEXT,
    contrast_a400=color.DARK_PRIMARY_TEXT,
    contrast_a700=color.DARK_PRIMARY_TEXT,
)


TEAL = ColorPalette(
    name="teal",
    color_50="#e0f2f1",
    color_100="#b2dfdb",
    color_200="#80cbc4",
    color_300="#4db6ac",
    color_400="#26a69a",
    color_500="#009688",
    color_600="#00897b",
    color_700="#00796b",
    color_800="#00695c",
    color_900="#004d40",
    color_a100="#a7ffeb",
    color_a200="#64ffda",
    color_a400="#1de9b6",
    color_a700="#00bfa5",
    contrast_50=color.DARK_PRIMARY_TEXT,
    contrast_100=color.DARK_PRIMARY_TEXT,
    contrast_200=color.DARK_PRIMARY_TEXT,
    contrast_300=color.DARK_PRIMARY_TEXT,
    contrast_400=color.DARK_PRIMARY_TEXT,
    contrast_500=color.LIGHT_PRIMARY_TEXT,
    contrast_600=color.LIGHT_PRIMARY_TEXT,
    contrast_700=color.LIGHT_PRIMARY_TEXT,
    contrast_800=color.LIGHT_PRIMARY_TEXT,
    contrast_900=color.LIGHT_PRIMARY_TEXT,
    contrast_a100=color.DARK_PRIMARY_TEXT,
    contrast_a200=color.DARK_PRIMARY_TEXT,
    contrast_a400=color.DARK_PRIMARY_TEXT,
    contrast_a700=color.DARK_PRIMARY_TEXT,
)


GREEN = ColorPalette(
    name="green",
    color_50="#e8f5e9",
    color_100="#c8e6c9",
    color_200="#a5d6a7",
    color_300="#81c784",
    color_400="#66bb6a",
    color_500="#4caf50",
    color_600="#43a047",
    color_700="#388e3c",
    color_800="#2e7d32",
    color_900="#1b5e20",
    color_a100="#b9f6ca",
    color_a200="#69f0ae",
    color_a400="#00e676",
    color_a700="#00c853",
    contrast_50=color.DARK_PRIMARY_TEXT,
    contrast_100=color.DARK_PRIMARY_TEXT,
    contrast_200=color.DARK_PRIMARY_TEXT,
    contrast_300=color.DARK_PRIMARY_TEXT,
    contrast_400=color.DARK_PRIMARY_TEXT,
    contrast_500=color.DARK_PRIMARY_TEXT,
    contrast_600=color.LIGHT_PRIMARY_TEXT,
    contrast_700=color.LIGHT_PRIMARY_TEXT,
    contrast_800=color.LIGHT_PRIMARY_TEXT,
    contrast_900=color.LIGHT_PRIMARY_TEXT,
    contrast_a100=color.DARK_PRIMARY_TEXT,
    contrast_a200=color.DARK_PRIMARY_TEXT,
    contrast_a400=color.DARK_PRIMARY_TEXT,
    contrast_a700=color.DARK_PRIMARY_TEXT,
)


LIGHT_GREEN = ColorPalette(
    name="light_green",
    color_50="#f1f8e9",
    color_100="#dcedc8",
    color_200="#c5e1a5",
    color_300="#aed581",
    color_400="#9ccc65",
    color_500="#8bc34a",
    color_600="#7cb342",
    color_700="#689f38",
    color_800="#558b2f",
    color_900="#33691e",
    color_a100="#ccff90",
    color_a200="#b2ff59",
    color_a400="#76ff03",
    color_a700="#64dd17",
    contrast_50=color.DARK_PRIMARY_TEXT,
    contrast_100=color.DARK_PRIMARY_TEXT,
    contrast_200=color.DARK_PRIMARY_TEXT,
    contrast_300=color.DARK_PRIMARY_TEXT,
    contrast_400=color.DARK_PRIMARY_TEXT,
    contrast_500=color.DARK_PRIMARY_TEXT,
    contrast_600=color.DARK_PRIMARY_TEXT,
    contrast_700=color.LIGHT_PRIMARY_TEXT,
    contrast_800=color.LIGHT_PRIMARY_TEXT,
    contrast_900=color.LIGHT_PRIMARY_TEXT,
    contrast_a100=color.DARK_PRIMARY_TEXT,
    contrast_a200=color.DARK_PRIMARY_TEXT,
    contrast_a400=color.DARK_PRIMARY_TEXT,
    contrast_a700=color.DARK_PRIMARY_TEXT,
)


LIME = ColorPalette(
    name="lime",
    color_50="#f9fbe7",
    color_100="#f0f4c3",
    color_200="#e6ee9c",
    color_300="#dce775",
    color_400="#d4e157",
    color_500="#cddc39",
    color_600="#c0ca33",
    color_700="#afb42b",
    color_800="#9e9d24",
    color_900="#827717",
    color_a100="#f4ff81",
    color_a200="#eeff41",
    color_a400="#c6ff00",
    color_a700="#aeea00",
    contrast_50=color.DARK_PRIMARY_TEXT,
    contrast_100=color.DARK_PRIMARY_TEXT,
    contrast_200=color.DARK_PRIMARY_TEXT,
    contrast_300=color.DARK_PRIMARY_TEXT,
    contrast_400=color.DARK_PRIMARY_TEXT,
    contrast_500=color.DARK_PRIMARY_TEXT,
    contrast_600=color.DARK_PRIMARY_TEXT,
    contrast_700=color.DARK_PRIMARY_TEXT,
    contrast_800=color.DARK_PRIMARY_TEXT,
    contrast_900=color.LIGHT_PRIMARY_TEXT,
    contrast_a100=color.DARK_PRIMARY_TEXT,
    contrast_a200=color.DARK_PRIMARY_TEXT,
    contrast_a400=color.DARK_PRIMARY_TEXT,
    contrast_a700=color.DARK_PRIMARY_TEXT,
)


YELLOW = ColorPalette(
    name="yellow",
    color_50="#fffde7",
    color_100="#fff9c4",
    color_200="#fff59d",
    color_300="#fff176",
    color_400="#ffee58",
    color_500="#ffeb3b",
    color_600="#fdd835",
    color_700="#fbc02d",
    color_800="#f9a825",
    color_900="#f57f17",
    color_a100="#ffff8d",
    color_a200="#ffff00",
    color_a400="#ffea00",
    color_a700="#ffd600",
    contrast_50=color.DARK_PRIMARY_TEXT,
    contrast_100=color.DARK_PRIMARY_TEXT,
    contrast_200=color.DARK_PRIMARY_TEXT,
    contrast_300=color.DARK_PRIMARY_TEXT,
    contrast_400=color.DARK_PRIMARY_TEXT,
    contrast_500=color.DARK_PRIMARY_TEXT,
    contrast_600=color.DARK_PRIMARY_TEXT,
    contrast_700=color.DARK_PRIMARY_TEXT,
    contrast_800=color.DARK_PRIMARY_TEXT,
    contrast_900=color.DARK_PRIMARY_TEXT,
    contrast_a100=color.DARK_PRIMARY_TEXT,
    contrast_a200=color.DARK_PRIMARY_TEXT,
    contrast_a400=color.DARK_PRIMARY_TEXT,
    contrast_a700=color.DARK_PRIMARY_TEXT,
)


AMBER = ColorPalette(
    name="amber",
    color_50="#fff8e1",
    color_100="#ffecb3",
    color_200="#ffe082",
    color_300="#ffd54f",
    color_400="#ffca28",
    color_500="#ffc107",
    color_600="#ffb300",
    color_700="#ffa000",
    color_800="#ff8f00",
    color_900="#ff6f00",
    color_a100="#ffe57f",
    color_a200="#ffd740",
    color_a400="#ffc400",
    color_a700="#ffab00",
    contrast_50=color.DARK_PRIMARY_TEXT,
    contrast_100=color.DARK_PRIMARY_TEXT,
    contrast_200=color.DARK_PRIMARY_TEXT,
    contrast_300=color.DARK_PRIMARY_TEXT,
    contrast_400=color.DARK_PRIMARY_TEXT,
    contrast_500=color.DARK_PRIMARY_TEXT,
    contrast_600=color.DARK_PRIMARY_TEXT,
    contrast_700=color.DARK_PRIMARY_TEXT,
    contrast_800=color.DARK_PRIMARY_TEXT,
    contrast_900=color.DARK_PRIMARY_TEXT,
    contrast_a100=color.DARK_PRIMARY_TEXT,
    contrast_a200=color.DARK_PRIMARY_TEXT,
    contrast_a400=color.DARK_PRIMARY_TEXT,
    contrast_a700=color.DARK_PRIMARY_TEXT,
)


ORANGE = ColorPalette(
    name="orange",
    color_50="#fff3e0",
    color_100="#ffe0b2",
    color_200="#ffcc80",
    color_300="#ffb74d",
    color_400="#ffa726",
    color_500="#ff9800",
    color_600="#fb8c00",
    color_700="#f57c00",
    color_800="#ef6c00",
    color_900="#e65100",
    color_a100="#ffd180",
    color_a200="#ffab40",
    color_a400="#ff9100",
    color_a700="#ff6d00",
    contrast_50=color.DARK_PRIMARY_TEXT,
    contrast_100=color.DARK_PRIMARY_TEXT,
    contrast_200=color.DARK_PRIMARY_TEXT,
    contrast_300=color.DARK_PRIMARY_TEXT,
    contrast_400=color.DARK_PRIMARY_TEXT,
    contrast_500=color.DARK_PRIMARY_TEXT,
    contrast_600=color.DARK_PRIMARY_TEXT,
    contrast_700=color.DARK_PRIMARY_TEXT,
    contrast_800=color.LIGHT_PRIMARY_TEXT,
    contrast_900=color.LIGHT_PRIMARY_TEXT,
    contrast_a100=color.DARK_PRIMARY_TEXT,
    contrast_a200=color.DARK_PRIMARY_TEXT,
    contrast_a400=color.DARK_PRIMARY_TEXT,
    contrast_a700=color.DARK_PRIMARY_TEXT,
)


DEEP_ORANGE = ColorPalette(
    name="deep_orange",
    color_50="#fbe9e7",
    color_100="#ffccbc",
    color_200="#ffab91",
    color_300="#ff8a65",
    color_400="#ff7043",
    color_500="#ff5722",
    color_600="#f4511e",
    color_700="#e64a19",
    color_800="#d84315",
    color_900="#bf360c",
    color_a100="#ff9e80",
    color_a200="#ff6e40",
    color_a400="#ff3d00",
    color_a700="#dd2c00",
    contrast_50=color.DARK_PRIMARY_TEXT,
    contrast_100=color.DARK_PRIMARY_TEXT,
    contrast_200=color.DARK_PRIMARY_TEXT,
    contrast_300=color.DARK_PRIMARY_TEXT,
    contrast_400=color.DARK_PRIMARY_TEXT,
    contrast_500=color.LIGHT_PRIMARY_TEXT,
    contrast_600=color.LIGHT_PRIMARY_TEXT,
    contrast_700=color.LIGHT_PRIMARY_TEXT,
    contrast_800=color.LIGHT_PRIMARY_TEXT,
    contrast_900=color.LIGHT_PRIMARY_TEXT,
    contrast_a100=color.DARK_PRIMARY_TEXT,
    contrast_a200=color.DARK_PRIMARY_TEXT,
    contrast_a400=color.LIGHT_PRIMARY_TEXT,
    contrast_a700=color.LIGHT_PRIMARY_TEXT,
)


BROWN = ColorPalette(
    name="brown",
    color_50="#efebe9",
    color_100="#d7ccc8",
    color_200="#bcaaa4",
    color_300="#a1887f",
    color_400="#8d6e63",
    color_500="#795548",
    color_600="#6d4c41",
    color_700="#5d4037",
    color_800="#4e342e",
    color_900="#3e2723",
    color_a100="#d7ccc8",
    color_a200="#bcaaa4",
    color_a400="#8d6e63",
    color_a700="#5d4037",
    contrast_50=color.DARK_PRIMARY_TEXT,
    contrast_100=color.DARK_PRIMARY_TEXT,
    contrast_200=color.DARK_PRIMARY_TEXT,
    contrast_300=color.LIGHT_PRIMARY_TEXT,
    contrast_400=color.LIGHT_PRIMARY_TEXT,
    contrast_500=color.LIGHT_PRIMARY_TEXT,
    contrast_600=color.LIGHT_PRIMARY_TEXT,
    contrast_700=color.LIGHT_PRIMARY_TEXT,
    contrast_800=color.LIGHT_PRIMARY_TEXT,
    contrast_900=color.LIGHT_PRIMARY_TEXT,
    contrast_a100=color.DARK_PRIMARY_TEXT,
    contrast_a200=color.DARK_PRIMARY_TEXT,
    contrast_a400=color.LIGHT_PRIMARY_TEXT,
    contrast_a700=color.LIGHT_PRIMARY_TEXT,
)


GREY = ColorPalette(
    name="grey",
    color_50="#fafafa",
    color_100="#f5f5f5",
    color_200="#eeeeee",
    color_300="#e0e0e0",
    color_400="#bdbdbd",
    color_500="#9e9e9e",
    color_600="#757575",
    color_700="#616161",
    color_800="#424242",
    color_900="#212121",
    color_a100="#ffffff",
    color_a200="#eeeeee",
    color_a400="#bdbdbd",
    color_a700="#616161",
    contrast_50=color.DARK_PRIMARY_TEXT,
    contrast_100=color.DARK_PRIMARY_TEXT,
    contrast_200=color.DARK_PRIMARY_TEXT,
    contrast_300=color.DARK_PRIMARY_TEXT,
    contrast_400=color.DARK_PRIMARY_TEXT,
    contrast_500=color.DARK_PRIMARY_TEXT,
    contrast_600=color.LIGHT_PRIMARY_TEXT,
    contrast_700=color.LIGHT_PRIMARY_TEXT,
    contrast_800=color.LIGHT_PRIMARY_TEXT,
    contrast_900=color.LIGHT_PRIMARY_TEXT,
    contrast_a100=color.DARK_PRIMARY_TEXT,
    contrast_a200=color.DARK_PRIMARY_TEXT,
    contrast_a400=color.DARK_PRIMARY_TEXT,
    contrast_a700=color.LIGHT_PRIMARY_TEXT,
)


BLUE_GREY = ColorPalette(
    name="blue_grey",
    color_50="#eceff1",
    color_100="#cfd8dc",
    color_200="#b0bec5",
    color_300="#90a4ae",
    color_400="#78909c",
    color_500="#607d8b",
    color_600="#546e7a",
    color_700="#455a64",
    color_800="#37474f",
    color_900="#263238",
    color_a100="#cfd8dc",
    color_a200="#b0bec5",
    color_a400="#78909c",
    color_a700="#455a64",
    contrast_50=color.DARK_PRIMARY_TEXT,
    contrast_100=color.DARK_PRIMARY_TEXT,
    contrast_200=color.DARK_PRIMARY_TEXT,
    contrast_300=color.DARK_PRIMARY_TEXT,
    contrast_400=color.LIGHT_PRIMARY_TEXT,
    contrast_500=color.LIGHT_PRIMARY_TEXT,
    contrast_600=color.LIGHT_PRIMARY_TEXT,
    contrast_700=color.LIGHT_PRIMARY_TEXT,
    contrast_800=color.LIGHT_PRIMARY_TEXT,
    contrast_900=color.LIGHT_PRIMARY_TEXT,
    contrast_a100=color.DARK_PRIMARY_TEXT,
    contrast_a200=color.DARK_PRIMARY_TEXT,
    contrast_a400=color.LIGHT_PRIMARY_TEXT,
    contrast_a700=color.LIGHT_PRIMARY_TEXT,
)

ORSTED_AQUA = ColorPalette(
    name="orsted_aqua",
    color_50="#f1f9f8",
    color_100="#ddf0ef",
    color_200="#c7e6e4",
    color_300="#b0dcd9",
    color_400="#9fd5d0",
    color_500="#8ecdc8",
    color_600="#86c8c2",
    color_700="#7bc1bb",
    color_800="#71bab4",
    color_900="#5faea7",
    color_a100="#ffffff",
    color_a200="#e8fffd",
    color_a400="#b5fff8",
    color_a700="#9cfff6",
    contrast_50=color.DARK_PRIMARY_TEXT,
    contrast_100=color.DARK_PRIMARY_TEXT,
    contrast_200=color.DARK_PRIMARY_TEXT,
    contrast_300=color.DARK_PRIMARY_TEXT,
    contrast_400=color.DARK_PRIMARY_TEXT,
    contrast_500=color.DARK_PRIMARY_TEXT,
    contrast_600=color.DARK_PRIMARY_TEXT,
    contrast_700=color.DARK_PRIMARY_TEXT,
    contrast_800=color.DARK_PRIMARY_TEXT,
    contrast_900=color.DARK_PRIMARY_TEXT,
    contrast_a100=color.DARK_PRIMARY_TEXT,
    contrast_a200=color.DARK_PRIMARY_TEXT,
    contrast_a400=color.DARK_PRIMARY_TEXT,
    contrast_a700=color.DARK_PRIMARY_TEXT,
)

ORSTED_AUBERGINE = ColorPalette(
    name="orsted aubergine",
    color_50="#eceaef",
    color_100="#d1c9d6",
    color_200="#b2a6bb",
    color_300="#93829f",
    color_400="#7b678b",
    color_500="#644c76",
    color_600="#5c456e",
    color_700="#523c63",
    color_800="#483359",
    color_900="#362446",
    color_a100="#ca8dff",
    color_a200="#b25aff",
    color_a400="#9a27ff",
    color_a700="#8e0eff",
    contrast_50=color.DARK_PRIMARY_TEXT,
    contrast_100=color.DARK_PRIMARY_TEXT,
    contrast_200=color.DARK_PRIMARY_TEXT,
    contrast_300=color.DARK_PRIMARY_TEXT,
    contrast_400=color.LIGHT_PRIMARY_TEXT,
    contrast_500=color.LIGHT_PRIMARY_TEXT,
    contrast_600=color.LIGHT_PRIMARY_TEXT,
    contrast_700=color.LIGHT_PRIMARY_TEXT,
    contrast_800=color.LIGHT_PRIMARY_TEXT,
    contrast_900=color.LIGHT_PRIMARY_TEXT,
    contrast_a100=color.DARK_PRIMARY_TEXT,
    contrast_a200=color.DARK_PRIMARY_TEXT,
    contrast_a400=color.LIGHT_PRIMARY_TEXT,
    contrast_a700=color.LIGHT_PRIMARY_TEXT,
)

ORSTED_BLUE = ColorPalette(
    name="orsted blue",
    color_50="#e8f3fb",
    color_100="#c6e0f4",
    color_200="#a0cced",
    color_300="#79b8e5",
    color_400="#5da8e0",
    color_500="#4099da",
    color_600="#3a91d6",
    color_700="#3286d0",
    color_800="#2a7ccb",
    color_900="#1c6bc2",
    color_a100="#f8fbff",
    color_a200="#c5dfff",
    color_a400="#92c2ff",
    color_a700="#79b4ff",
    contrast_50=color.DARK_PRIMARY_TEXT,
    contrast_100=color.DARK_PRIMARY_TEXT,
    contrast_200=color.DARK_PRIMARY_TEXT,
    contrast_300=color.DARK_PRIMARY_TEXT,
    contrast_400=color.DARK_PRIMARY_TEXT,
    contrast_500=color.DARK_PRIMARY_TEXT,
    contrast_600=color.LIGHT_PRIMARY_TEXT,
    contrast_700=color.LIGHT_PRIMARY_TEXT,
    contrast_800=color.LIGHT_PRIMARY_TEXT,
    contrast_900=color.LIGHT_PRIMARY_TEXT,
    contrast_a100=color.DARK_PRIMARY_TEXT,
    contrast_a200=color.DARK_PRIMARY_TEXT,
    contrast_a400=color.DARK_PRIMARY_TEXT,
    contrast_a700=color.DARK_PRIMARY_TEXT,
)

ORSTED_SUN = ColorPalette(
    name="orsted sun",
    color_50="fcebeb",
    color_100="f8cdcd",
    color_200="f4abab",
    color_300="ef8989",
    color_400="eb7070",
    color_500="e85757",
    color_600="e54f4f",
    color_700="e24646",
    color_800="de3c3c",
    color_900="d82c2c",
    color_a100="ffffff",
    color_a200="ffe5e5",
    color_a400="ffb3b3",
    color_a700="ff9999",
    contrast_50=color.DARK_PRIMARY_TEXT,
    contrast_100=color.DARK_PRIMARY_TEXT,
    contrast_200=color.DARK_PRIMARY_TEXT,
    contrast_300=color.DARK_PRIMARY_TEXT,
    contrast_400=color.DARK_PRIMARY_TEXT,
    contrast_500=color.DARK_PRIMARY_TEXT,
    contrast_600=color.LIGHT_PRIMARY_TEXT,
    contrast_700=color.LIGHT_PRIMARY_TEXT,
    contrast_800=color.LIGHT_PRIMARY_TEXT,
    contrast_900=color.LIGHT_PRIMARY_TEXT,
    contrast_a100=color.DARK_PRIMARY_TEXT,
    contrast_a200=color.DARK_PRIMARY_TEXT,
    contrast_a400=color.DARK_PRIMARY_TEXT,
    contrast_a700=color.DARK_PRIMARY_TEXT,
)

PALETTES = [
    AMBER,
    BLUE_GREY,
    BLUE,
    BROWN,
    CYAN,
    DEEP_ORANGE,
    DEEP_PURPLE,
    GREEN,
    GREY,
    INDIGO,
    LIGHT_BLUE,
    LIGHT_GREEN,
    LIME,
    ORANGE,
    PINK,
    PURPLE,
    RED,
    TEAL,
    YELLOW,
    ORSTED_AQUA,
    ORSTED_AUBERGINE,
    ORSTED_BLUE,
    ORSTED_SUN,
]



"""Implementation of MWC Material Components"""
import panel as pn
import param

from .config import MWC_ICONS
from .material_button import Button
from .material_extension import Extension
from .material_progress import CircularProgress, LinearProgress
from .material_select import Select
from .material_sliders import FloatSlider, IntSlider
from .style.stylesheet import Stylesheet
